/******************************************************************************
  (c) Copyright 2002,2003, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile: DOMSerializer.java,v $
  Version:       $Name:  $ $Revision: 1.2 $
  Last Modified: $Date: 2008/06/11 13:58:07 $
 *****************************************************************************/

package org.netkernel.xml.transrepresentation;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.netkernel.xml.representation.RepSerializedHTML;
import org.netkernel.xml.util.XMLUtils;
import org.netkernel.xml.xda.DOMXDA;
import org.w3c.dom.*;

import java.io.*;

/**
 * Serialise DOM to a binary stream
 * @author  tab
 */
public class DOMSerializer extends StandardTransreptorImpl
{
	public static final String META_ENCODING="encoding";
	
	public DOMSerializer()
	{	this.declareThreadSafe();
		this.declareName("DOMSerializer");
		this.declareDescription("serialize DOM to String or binary stream");
		this.declareFromRepresentation(Node.class);
		this.declareFromRepresentation(DOMXDA.class);
		this.declareToRepresentation(IReadableBinaryStreamRepresentation.class);
		this.declareToRepresentation(String.class);
		
	}
	
	public void onTransrept(INKFRequestContext aContext) throws Exception
	{	
		INKFResponseReadOnly<Object> responseIn=aContext.getThisRequest().getPrimaryAsResponse();
		Node n;
		if (responseIn.getRepresentation() instanceof Node)
		{	n=(Node)responseIn.getRepresentation();
		}
		else
		{	n=((DOMXDA)responseIn.getRepresentation()).getDocument();
		}
		
		String mimetype=responseIn.getMimeType();
		boolean isHTML = mimetype.equals("text/html");
		String encoding;
		if (responseIn.hasHeader(META_ENCODING))
		{	//encoding=(String)responseIn.getMetaDataAs(META_ENCODING, String.class);
			encoding=responseIn.getHeaderAs(META_ENCODING, String.class);
		}
		else
		{	encoding="UTF-8";
		}
		
		//StringWriter sw = new StringWriter(2048);
		
		Writer sw;
		ByteArrayOutputStream baos=null;
		
		if (aContext.getThisRequest().getRepresentationClass().isAssignableFrom(String.class))
		{	sw = new StringWriter(2048);
		}
		else
		{	baos = new ByteArrayOutputStream(2048);
			sw = new OutputStreamWriter(baos,encoding);
		}
		
		DocumentType dt = XMLUtils.getDocumentFor(n).getDoctype();
		sw.write("<?xml version=\"1.0\" encoding=\"");
		sw.write(encoding);
		sw.write("\"?>\n");
		if (dt!=null)
		{	writeDocumentType(sw,dt);
		}
		if (isHTML)
		{	RepSerializedHTML.serialize(n,sw);
		}
		else
		{	XMLUtils.toXML(sw, n, false,true,encoding);
		}
		sw.flush();
		
		Object rep;
		if (aContext.getThisRequest().getRepresentationClass().isAssignableFrom(String.class))
		{	rep=sw.toString();
		}
		else
		{	rep = new ByteArrayRepresentation(baos);
		}
		
		INKFResponse response = aContext.createResponseFrom(rep);
		if(mimetype==null || mimetype.equals("content/unknown"))
		{	response.setMimeType(isHTML?"text/html":"text/xml");
		}
	}	
	
	private void writeDocumentType(Writer sw, DocumentType dt) throws IOException
	{	sw.write("<!DOCTYPE ");
		sw.write(dt.getName());
		if (dt.getPublicId()!=null)
		{	sw.write(" PUBLIC \"");
			sw.write(dt.getPublicId());
			sw.write("\"");
		}
		if (dt.getSystemId()!=null)
		{	sw.write(" \"");
			sw.write(dt.getSystemId());
			sw.write("\"");
		}
		sw.write(">\n");
	}
}