package org.netkernel.xml.transrepresentation;

import java.lang.reflect.Array;
import java.util.*;
import java.beans.*;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResolutionContext;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IRepAny;
import org.netkernel.layer0.tools.ExtraMimeTypes;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.netkernel.xml.representation.IBeanFromXML;
import org.netkernel.xml.xda.*;



public class XMLToBean extends StandardTransreptorImpl
{
	public XMLToBean()
	{	declareThreadSafe();
		declareToRepresentation(IRepAny.class);
		declareToRepresentation(IBeanFromXML.class);
		declareDescription("initialise a bean based on an XML definition");
	}
	
	public void onTransrept(INKFRequestContext aContext) throws Exception
	{
		IXDAReadOnly xda = (IXDAReadOnly)aContext.sourcePrimary(IXDAReadOnly.class);
		Class to = aContext.getThisRequest().getRepresentationClass();
		
		IBeanFromXML instance = createBean(xda,to);
		instance.assertInvariant();
		aContext.createResponseFrom(instance);
	}
	
	private IBeanFromXML createBean(IXDAReadOnly aXML, Class aType) throws Exception
	{	
		IBeanFromXML instance = (IBeanFromXML)aType.newInstance();
		
		BeanInfo bi = Introspector.getBeanInfo(aType,Object.class);
		PropertyDescriptor mds[] = bi.getPropertyDescriptors();
		for (int i=0; i<mds.length; i++)
		{	PropertyDescriptor desc = mds[i];
			boolean indexed = desc instanceof IndexedPropertyDescriptor;
			//System.out.println("looking at "+mds[i].getDisplayName()+" "+mds[i].getPropertyType().getName());
			if (!indexed)
			{	String xpath=desc.getShortDescription();
				String value;
				try
				{	value=aXML.getText(xpath,false);
					//System.out.println(value);
					//System.out.println(desc.getPropertyType().getCanonicalName());
					
					Object arg = constructFor(desc,value);
					desc.getWriteMethod().invoke(instance,new Object[] {arg});
					//System.out.println(arg);
				}
				catch (XPathLocationException e)
				{	// no value in the XML
				}
			}
			else
			{	String xpath=desc.getShortDescription()+"/*";
				Class pType = desc.getPropertyType().getComponentType();
				//System.out.println(xpath);
				List items = new ArrayList();
				for (IXDAReadOnlyIterator j=aXML.readOnlyIterator(xpath); j.hasNext(); )
				{	j.next();
					//System.out.println(j.toString());
					Object item = createBean(j,pType);
					items.add(item);
				}
				Object itemArray = Array.newInstance(pType,items.size());
				for (int k=0; k<items.size(); k++)
				{	Array.set(itemArray,k,items.get(k));
				}
				desc.getWriteMethod().invoke(instance,new Object[] {itemArray} );
			}
		}
		return instance;
	}
	
	private Object constructFor(PropertyDescriptor aDesc, String aValue)
	{	Class pType = aDesc.getPropertyType();
		Object arg;
		if (pType==String.class)
		{	arg = aValue;
		}
		else if (pType==Integer.TYPE)
		{	arg = new Integer(aValue);
		}
		else if (pType==Long.TYPE)
		{	arg = new Long(aValue);
		}
		else if (pType==Float.TYPE)
		{	arg = new Float(aValue);
		}
		else if (pType==Double.TYPE)
		{	arg = new Double(aValue);
		}
		else if (pType==Boolean.TYPE)
		{	if (aValue.length()==0)
			{	arg=Boolean.TRUE;
			}
			else
			{	arg = new Boolean(aValue);
			}
		}
		else
		{	throw new IllegalArgumentException("unsupported type: "+pType);
		}
		return arg;
	}

	protected boolean additionalResolveChecks(INKFResolutionContext aContext) throws Exception
	{	// confirm primary is indeed some form of xml
		INKFResponseReadOnly primary=aContext.getRequestToResolve().getPrimaryAsResponse();
		String mime=primary.getMimeType();
		return ExtraMimeTypes.getInstance().isAssignableAFromB("text/xml", mime);
	}
}
